//
//  CheckinController.swift
//  TIME
//
//  Created by Sruit Angkavanitsuk on 3/7/20.
//  Copyright © 2020 NSTDA. All rights reserved.
//

import UIKit

class CheckinController: UIViewController {
    
    fileprivate let saveStorage = UserDefaults.standard
    var tapGesture:UITapGestureRecognizer?
    var appDelegate:AppDelegate!
    var empID = ""
    private var dataGetEmp = ""
    var timeLostFocus: TimeInterval = 0
    var isBackFromController = false
    var idleCheck:DispatchWorkItem!
    
    @IBOutlet weak var ivLogo: UIImageView!
    @IBOutlet weak var ivHowto: UIImageView!
    @IBOutlet weak var vCheckin: UIView!
    @IBOutlet weak var btnClose: UIView!
    @IBOutlet weak var btnHowto: UIView!
    @IBOutlet weak var btnFAQ: UIView!
    
    fileprivate var loadingProgress : UIActivityIndicatorView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        appDelegate = UIApplication.shared.delegate as! AppDelegate
        if #available(iOS 13.0, *) {
            overrideUserInterfaceStyle = .light // force light mode
        }
        // Do any additional setup after loading the view.
        
        // load first time
        if let firstTimeCheckin  = saveStorage.string(forKey: "FIRST_TIME_CHECKIN") {
            ivHowto.isHidden = true
            ivLogo.isHidden = false
            vCheckin.isHidden = false
        }
        else {
            // first time, checkin
            // create tap gesture recognizer
            tapGesture = UITapGestureRecognizer(target: self, action: #selector(CheckinController.imageTapped(gesture:)))
            
            ivHowto.isHidden = false
            ivHowto.addGestureRecognizer(tapGesture!)
            ivHowto.isUserInteractionEnabled = true
            ivLogo.isHidden = true
            vCheckin.isHidden = true
            
            saveStorage.set("TRUE", forKey: "FIRST_TIME_CHECKIN")
        }
        
        idleCheck = DispatchWorkItem(block: {
            // if stay in this page longer than 300 seconds
            // refresh in 300 sec
            DataFactory.IS_LOADING_FIRSTTIME = true
            self.performSegue(withIdentifier: "segue_exit_checkin_to_loading", sender: nil)
        })

        NotificationCenter.default.addObserver(
            self,
            selector: #selector(applicationWillEnterForeground),
            name: UIApplication.willEnterForegroundNotification,
            object: nil)
        NotificationCenter.default.addObserver(
            self,
            selector: #selector(applicationWillResignActive),
            name: UIApplication.willResignActiveNotification,
            object: nil)
        
        // loading view
        if DataFactory.BUSINESS_CODE == 1{
            ivLogo.image = UIImage(named: "i_nectec_checkin_logo")
            ivHowto.image = UIImage(named: "i_nectec_checkin_howto")
        }
        else if DataFactory.BUSINESS_CODE == 2 {
            ivLogo.image = UIImage(named: "i_nbtc_checkin_logo")
            ivHowto.image = UIImage(named: "i_nbtc_checkin_howto")
        }
        
        //
        if(DataFactory.IS_CHECKIN_ALLOW == 0){
            let returnMessage = "กรุณาลงเวลาในพื้นที่ที่คุณมีสิทธ์"
            Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: returnMessage, isCloseApplication: true)
        }
        
        // loading animation
        loadingProgress = UIActivityIndicatorView() as UIActivityIndicatorView
        loadingProgress.frame = CGRect(x: 0, y: 0,
                                       width: UIScreen.main.bounds.size.width,
                                       height: UIScreen.main.bounds.size.height)
        loadingProgress.isOpaque = false
        loadingProgress.backgroundColor = UIColor( red: 0.95, green: 0.95, blue: 0.95, alpha: 0.5 )
        loadingProgress.hidesWhenStopped = true
        loadingProgress.style = UIActivityIndicatorView.Style.gray
        view.addSubview(loadingProgress)
        
    }
    
    override func viewWillAppear(_ animated: Bool) {
        NotificationCenter.default.addObserver(self, selector: #selector(keyboardWillShow), name: UIResponder.keyboardWillShowNotification, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(keyboardWillHide), name: UIResponder.keyboardWillHideNotification, object: nil)
        
        DispatchQueue.main.asyncAfter(deadline: .now() + 300, execute: idleCheck)
        checkTimeLostFocus()
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        NotificationCenter.default.removeObserver(self, name: UIResponder.keyboardWillShowNotification, object: self.view.window)
        NotificationCenter.default.removeObserver(self, name: UIResponder.keyboardWillHideNotification, object: self.view.window)
        
        idleCheck.cancel()
        //saveTimeLostFocus
        timeLostFocus = Date().timeIntervalSince1970
    }
    
    /**
     part: connect API
     */
    
    // status = 4
    func requestCheckinGetEmpIDFromMACAddress (){
        
        // data
        // nonokyo: set here
        let status = "4";
        let macAdr = appDelegate.UUID
        /*
        var business = ""
        if DataFactory.BUSINESS_CODE == 1 {
            business = "NECTEC"
        }
        else if DataFactory.BUSINESS_CODE == 2 {
            business = "NBTC"
        }*/
        dataGetEmp = "\(status)+getEmpID+\(macAdr)"
        
        if DataFactory.DEBUG {
            print("dataGetEmp=\(dataGetEmp)")
            print("macAdr=\(macAdr)")
        }
        // url
        // nonokyo: set here
        let url = DataFactory.REQUEST_API_ACCOUNT_CHECKIN_GET_EMPID_URL
        
        let request:NSMutableURLRequest = NSMutableURLRequest(url: URL(string: url)!)
        request.httpMethod = "POST"
        request.addValue("multipart/form-data; boundary=X", forHTTPHeaderField: "Content-Type")
        
        // post data
        // nonokyo: set here
        let postData = DataFactory.generatePostData(
            [DataFactory.GeneratePostObject(name: "getEmpID_data", value: EncryptDecryptAlgorithm.EncryptData(dataGetEmp)),
             DataFactory.GeneratePostObject(name: "getEmpIDK", value: EncryptDecryptAlgorithm.getKeyEncrypt())]
        )
        // display sent data
        if DataFactory.DEBUG {
            print("url=\(request.url!)")
            print("data=\n\(postData)")
        }
        request.httpBody = postData.data(using: String.Encoding.utf8)
        
        DispatchQueue.main.async(execute: { () -> Void in
            self.loadingProgress.startAnimating()
        })
        
        let session = URLSession.shared
        let task = session.dataTask(with: request as URLRequest) { (data, response, error) -> Void in
            
            if error != nil {
                DispatchQueue.main.async(execute: { () -> Void in
                    self.loadingProgress.stopAnimating()
                    Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: DataString.error_connection_timeout)
                })
                return
            }
            
            var returnResult = 0
            var returnMessage = ""
            let result = NSString(data: data!, encoding: String.Encoding.utf8.rawValue)! as String
            if DataFactory.DEBUG {
                print("result=\(result)")
            }
            
            if !result.isEmpty {
                if result.hasPrefix("<!DOCTYPE") {
                    returnMessage = "Error: return data corrupted, please contact administrator."
                }
                else {
                    self.empID = result
                    returnResult = 1
                }
            }
            else {
                returnMessage = "response == null"
                returnResult = 0
            }
            
            if returnResult == 1 {
                DispatchQueue.main.async(execute: { () -> Void in
                    self.loadingProgress.stopAnimating()
                    // set segue to new page here
                    // nonokyo: set here
                    self.performSegue(withIdentifier: "segue_checkin_to_captface", sender: nil)
                })
            }
            else {
                DispatchQueue.main.async(execute: { () -> Void in
                    self.loadingProgress.stopAnimating()
                    Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: returnMessage)
                })
            }
        }
        task.resume()
    }
    
    // status = ?
    func requestUpdateGetEmpIDFromMACAddress (){
        // data
        // nonokyo: set here
        let macAdr = appDelegate.UUID
        let status = "update1";
        let datatwo = "\(status)+\(macAdr)";
        
        if DataFactory.DEBUG {
            print("dataforth=\(datatwo)")
        }
        // url
        // nonokyo: set here
        let url = DataFactory.REQUEST_API_ACCOUNT_UPDATE_GET_EMPID_URL
        
        let request:NSMutableURLRequest = NSMutableURLRequest(url: URL(string: url)!)
        request.httpMethod = "POST"
        request.addValue("multipart/form-data; boundary=X", forHTTPHeaderField: "Content-Type")
        
        // post data
        // nonokyo: set here
        let postData = DataFactory.generatePostData(
            [DataFactory.GeneratePostObject(name: "data_two", value: EncryptDecryptAlgorithm.EncryptData(datatwo)),
             DataFactory.GeneratePostObject(name: "data_twoK", value: EncryptDecryptAlgorithm.getKeyEncrypt())]
        )
        // display sent data
        if DataFactory.DEBUG {
            print("url=\(request.url!)")
            print("data=\n\(postData)")
        }
        request.httpBody = postData.data(using: String.Encoding.utf8)
        
        DispatchQueue.main.async(execute: { () -> Void in
            self.loadingProgress.startAnimating()
        })
        
        let session = URLSession.shared
        let task = session.dataTask(with: request as URLRequest) { (data, response, error) -> Void in
            
            if error != nil {
                DispatchQueue.main.async(execute: { () -> Void in
                    self.loadingProgress.stopAnimating()
                    Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: DataString.error_connection_timeout)
                })
                return
            }
            
            var returnResult = 0
            var returnMessage = ""
            let result = NSString(data: data!, encoding: String.Encoding.utf8.rawValue)! as String
            if DataFactory.DEBUG {
                print("result=\(result)")
            }
            
            if !result.isEmpty {
                // ========== RESPONSE UNKNOWN / DATA CORRUPTED ===========
                if result.hasPrefix("<!DOCTYPE") {
                    returnMessage = "Error: return data corrupted, please contact administrator."
                    returnResult = 0
                }
                // ========== RESPONSE SUCCESS ===========
                else {
                    self.empID = result
                    returnResult = 1
                }
            }
            else {
                returnMessage = "response == null"
                returnResult = 0
            }
            
            if returnResult == 1 {
                DispatchQueue.main.async(execute: { () -> Void in
                    self.loadingProgress.stopAnimating()
                    // set segue to new page here
                    // nonokyo: set here
                    self.performSegue(withIdentifier: "segue_checkin_to_update", sender: nil)
                })
            }
            else {
                DispatchQueue.main.async(execute: { () -> Void in
                    self.loadingProgress.stopAnimating()
                    Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: returnMessage)
                })
            }
        }
        task.resume()
    }
    
    // status = 9
    func requestRemoveGetEmpIDFromMACAddress(){
        // data
        // nonokyo: set here
        let macAdr = appDelegate.UUID
        let status = "9"; // status delete = 9
        let datatwo = "\(status)+\(macAdr)";
        // url
        // nonokyo: set here
        let url = DataFactory.REQUEST_API_ACCOUNT_DELETE_GET_EMPID_URL
        
        let request:NSMutableURLRequest = NSMutableURLRequest(url: URL(string: url)!)
        request.httpMethod = "POST"
        request.addValue("multipart/form-data; boundary=X", forHTTPHeaderField: "Content-Type")
        
        // post data
        // nonokyo: set here
        let postData = DataFactory.generatePostData(
            [DataFactory.GeneratePostObject(name: "two_data", value: EncryptDecryptAlgorithm.EncryptData(datatwo)),
             DataFactory.GeneratePostObject(name: "twoK", value: EncryptDecryptAlgorithm.getKeyEncrypt())]
        )
        // display sent data
        if DataFactory.DEBUG {
            print("url=\(request.url!)")
            print("data=\n\(postData)")
        }
        request.httpBody = postData.data(using: String.Encoding.utf8)
        
        DispatchQueue.main.async(execute: { () -> Void in
            self.loadingProgress.startAnimating()
        })
        
        let session = URLSession.shared
        let task = session.dataTask(with: request as URLRequest) { (data, response, error) -> Void in
            
            if error != nil {
                DispatchQueue.main.async(execute: { () -> Void in
                    self.loadingProgress.stopAnimating()
                    Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: DataString.error_connection_timeout)
                })
                return
            }
            
            var returnResult = 0
            var returnMessage = ""
            let result = NSString(data: data!, encoding: String.Encoding.utf8.rawValue)! as String
            
            if !result.isEmpty {
                // ========== RESPONSE UNKNOWN / DATA CORRUPTED ===========
                if result.hasPrefix("<!DOCTYPE") {
                    returnMessage = "Error: return data corrupted, please contact administrator."
                    returnResult = 0
                }
                // ========== RESPONSE SUCCESS ===========
                else {
                    self.empID = result
                    returnResult = 1
                }
            }
            else {
                returnMessage = "response == null"
                returnResult = 0
            }
            
            if returnResult == 1 {
                DispatchQueue.main.async(execute: { () -> Void in
                    self.loadingProgress.stopAnimating()
                    // set segue to new page here
                    // nonokyo: set here
                    self.performSegue(withIdentifier: "segue_checkin_to_remove", sender: nil)
                })
                
            }
            else {
                DispatchQueue.main.async(execute: { () -> Void in
                    self.loadingProgress.stopAnimating()
                    Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: returnMessage)
                })
            }
        }
        task.resume()
    }
    
    
    /**
     part: etc
     */
    @IBAction func bCheckinClicked(_ sender: UIButton) {
        
        if DataFactory.IS_CHECKIN_ALLOW > 0 {
            requestCheckinGetEmpIDFromMACAddress()
        }
        else {
            let returnMessage = "กรุณาลงเวลาในพื้นที่ที่คุณมีสิทธ์"
            Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: returnMessage, isCloseApplication: true)
        }
    }
    
    @IBAction func bUpdateClicked(_ sender: UIButton) {
        requestUpdateGetEmpIDFromMACAddress()
    }
    
    @IBAction func bRemoveClicked(_ sender: UIButton) {
        requestRemoveGetEmpIDFromMACAddress()
    }
    
    @IBAction func bCloseClicked(_ sender: UIButton) {
        ivLogo.isHidden = false
        vCheckin.isHidden = false
        btnFAQ.isHidden = false
        btnHowto.isHidden = false
        
        ivHowto.isHidden = true
        btnClose.isHidden = true
    }
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "segue_checkin_to_captface" {
            let captureFaceController = segue.destination as! CaptureFaceController
            captureFaceController.from_code = "CHECKIN"
            captureFaceController.empID = self.empID
        }
        else if segue.identifier == "segue_checkin_to_update" {
            let updateController = segue.destination as! UpdateController
            updateController.empID = self.empID
        }
        else if segue.identifier == "segue_checkin_to_remove" {
            let removeController = segue.destination as! RemoveController
            removeController.empID = self.empID
        }
    }
    
    @objc func imageTapped(gesture: UIGestureRecognizer) {
        // if the tapped view is a UIImageView then set it to imageview
        if (gesture.view as? UIImageView) != nil &&
            gesture.view == ivHowto {
            ivHowto.removeGestureRecognizer(tapGesture!)
            ivHowto.isHidden = true
            ivLogo.isHidden = false
            vCheckin.isHidden = false
        }
    }
    
    @IBAction func segue_exit_to_checkin(_ segue: UIStoryboardSegue) {
        let segueName = segue.identifier
        if segueName == "segue_exit_captface_to_checkin" ||
            segueName == "segue_exit_update_to_checkin" ||
            segueName == "segue_exit_remove_to_checkin" {
            isBackFromController = true
            /*
            if let captFaceController = segue.source as? CaptureFaceController {
                //self.view.makeToast("ยืนยันตัวตนเรียบร้อย", duration: 3.0, position: .center)
            }
            */
        }
    }
    
    @IBAction func bFAQClicked(_ sender: UIButton) {
        /*var discTxt = "วัตถุประสงค์นี้ใช้เพื่อสำหรับการลงเวลาด้วยวิธีการตรวจสอบปัจจัยที่หลากหลายได้แก่ ใบหน้า รหัสผ่าน โทรศัพท์มือถือและสถานที่ ระบบนี้จะมีการบันทึกข้อมูลใบหน้าของผู้ใช้งาน และข้อมูลใบหน้าอาจถูกนำมาใช้เพื่อปรับปรุงระบบให้ดียิ่งขึ้นต่อไปในอนาคตและจะไม่มีการนำข้อมูลใบหน้าของผู้ใช้งานไปใช้ในวัตถุประสงค์อื่นนอกเหนือจากงานวิจัยภานใน ปัจจุบันระบบนี้ยังอยู่ระหว่างทดลองการใช้งาน"
        
        let alert = UIAlertController(title: "Disclamer", message: discTxt, preferredStyle: .alert)
        alert.addAction(UIAlertAction(title: "รับทราบ", style: .cancel, handler: nil))

        self.present(alert, animated: true)*/
        
        let urlComponents = URLComponents (string: "http://attime.secteam.in.th/#faq")!
        if #available(iOS 10.0, *) {
            UIApplication.shared.open(urlComponents.url!, options: [:], completionHandler: nil)
        } else {
            UIApplication.shared.openURL(urlComponents.url!)
        }
 
    }
    
    @IBAction func bHowtoClicked(_ sender: UIButton){
        
        // loading view
        if DataFactory.BUSINESS_CODE == 1{
            ivLogo.image = UIImage(named: "i_nectec_checkin_logo")
            ivHowto.image = UIImage(named: "i_nectec_checkin_howto")
        }
        else if DataFactory.BUSINESS_CODE == 2 {
            ivLogo.image = UIImage(named: "i_nbtc_checkin_logo")
            ivHowto.image = UIImage(named: "i_nbtc_checkin_howto")
        }
        
        var isFirstTime = true
        if isFirstTime {
            
            // create tap gesture recognizer
            tapGesture = UITapGestureRecognizer(target: self, action: #selector(CheckinController.imageTapped(gesture:)))
            
            ivHowto.isHidden = false
            ivHowto.addGestureRecognizer(tapGesture!)
            ivHowto.isUserInteractionEnabled = true
            ivLogo.isHidden = true
            vCheckin.isHidden = true
            
            btnFAQ.isHidden = true
            btnClose.isHidden = false
            btnHowto.isHidden = true
        }
        else {
            ivHowto.isHidden = true
            ivLogo.isHidden = false
            vCheckin.isHidden = false
        }
        
    }
    
    /**
     * Called when the user click on the view (outside the UITextField).
     */
    override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?) {
        self.view.endEditing(true)
    }
    
    @objc func keyboardWillShow(notification: NSNotification) {
        if let keyboardSize = (notification.userInfo?[UIResponder.keyboardFrameEndUserInfoKey] as? NSValue)?.cgRectValue {
            if self.view.frame.origin.y == 0 {
                self.view.frame.origin.y -= keyboardSize.height
            }
        }
    }
    
    @objc func keyboardWillHide(notification: NSNotification) {
        if self.view.frame.origin.y != 0 {
            self.view.frame.origin.y = 0
        }
    }
    
    @objc func applicationWillEnterForeground() {
        NSLog("CheckinController enter focus")
        checkTimeLostFocus()
    }
    
    @objc func applicationWillResignActive() {
        NSLog("CheckinController lost focus")
        //saveTimeLostFocus
        timeLostFocus = Date().timeIntervalSince1970
    }
    
    func checkTimeLostFocus() {
        NSLog("timeLostFocus = \(timeLostFocus)")
        NSLog("Date().timeIntervalSince1970 = \(Date().timeIntervalSince1970)")
        
        if (isBackFromController &&
            timeLostFocus > 0 &&
            timeLostFocus + 300 < Date().timeIntervalSince1970) {
            // come back from update, delete, other page
            // refresh in 300 sec
            DataFactory.IS_LOADING_FIRSTTIME = true
            self.performSegue(withIdentifier: "segue_exit_checkin_to_loading", sender: nil)
        }
        else if (!isBackFromController &&
            timeLostFocus > 0 &&
            timeLostFocus + 60 < Date().timeIntervalSince1970) {
            // switch to another app and come back
            // refresh in 60 sec
            DataFactory.IS_LOADING_FIRSTTIME = true
            self.performSegue(withIdentifier: "segue_exit_checkin_to_loading", sender: nil)
        }
    }
    
    deinit {
        NotificationCenter.default.removeObserver(self)
    }
}

