//
//  ViewController.swift
//  TIME
//
//  Created by Sruit Angkavanitsuk on 2/7/20.
//  Copyright © 2020 NSTDA. All rights reserved.
//

import UIKit
import AVFoundation
import CoreLocation
import SystemConfiguration.CaptiveNetwork

class LoadingController: UIViewController, CLLocationManagerDelegate {
    
    var appDelegate: AppDelegate!
    var wifiTryTimer: Timer?
    var isReceiveWifiBSSID = false
    var isCheckAgreement = ""
    
    /*
     // this page no need to have loading progress
     fileprivate var loadingProgress : UIActivityIndicatorView!
     */
    fileprivate let saveStorage = UserDefaults.standard
    fileprivate let systemSoundID: SystemSoundID = 1104
    
    @IBOutlet weak var indicator: UIActivityIndicatorView!
    
    @IBAction func segue_exit_to_loading(_ segue:UIStoryboardSegue){}
   
    var locationManager = CLLocationManager()
    var currentNetworkInfos: Array<NetworkInfo>? {
        get {
            return SSID.fetchNetworkInfo()
        }
    }

    override func viewDidLoad() {
        super.viewDidLoad()
        appDelegate = UIApplication.shared.delegate as? AppDelegate
        if #available(iOS 13.0, *) {
            overrideUserInterfaceStyle = .light // force light mode
        }
        // Do any additional setup after loading the view.
         
        // iOS >= 13
        if #available(iOS 13.0, *) {
            // setup UI
            indicator.style = .large
            
            // get location, wifi bssid
            let status = locationAuthorizationStatus()
            // 20210422
            // CLLocationManager.authorizationStatus() has deprecate in iOS 14.0
            //let status = CLLocationManager.authorizationStatus()
            if status == .authorizedWhenInUse {
                updateWiFiInformation()
            } else {
                locationManager.delegate = self
                locationManager.requestWhenInUseAuthorization()
            }
            
        }
        // iOS <= 12
        else {
            // setup UI
            indicator.style = .whiteLarge
            
            // get location, wifi bssid
            updateWiFiInformation()
        }

        // for test
        
        // test check-mac
        /*
         // http://203.185.96.59:8080/user/CheckMac
         var test = "5+\(appDelegate.UUID)"
         var data1 = EncryptDecryptAlgorithm.EncryptData(test)
         var data2 = EncryptDecryptAlgorithm.getKeyEncrypt()
         var end = "yo"
         */
        
        // test request delete - passed
        /*
         //http://203.185.96.59:8080/user/LoginDelete
         var test = "9+\(appDelegate.UUID)"
         var data1 = EncryptDecryptAlgorithm.EncryptData(test)
         var data2 = EncryptDecryptAlgorithm.getKeyEncrypt()
         var end = "yo"
         */
        
        // test confirm password delete - passed
        /*
         //http://203.185.96.59:8080/user/LoginDeletePushBtn
         var test = "10+toon+123456"
         var data1 = EncryptDecryptAlgorithm.EncryptData(test)
         var data2 = EncryptDecryptAlgorithm.getKeyEncrypt()
         var test3 = "123456"
         var data3 = EncryptDecryptAlgorithm.EncryptData(test3)
         var end = "yo"
         */
        
        // test confirm delete - passed
        /*
         //http://203.185.96.59:8080/user/Delete
         var test = "11+toon"
         var data1 = EncryptDecryptAlgorithm.EncryptData(test)
         var data2 = EncryptDecryptAlgorithm.getKeyEncrypt()
         var end = "yo"
         */
        
        
        /*
         var k = "BOHLAOAKSO+2020-7-3"
         do {
         let str = "helloworld"
         
         //let message: String = "Hi, Ross"
         let clear = try ClearMessage(string: str, using: .utf8)
         let encryptedMessage = try clear.encrypted(with: k, padding: .PKCS1)
         let encryptedMessageString = encryptedMessage.base64String
         
         print(encryptedMessageString)
         }
         catch {
         
         print(error)
         }
         */
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)

        if DataFactory.IS_LOADING_FIRSTTIME {

            // case not found bssid, will try 5 times, before exit app
            if !isReceiveWifiBSSID &&
                !DataFactory.DEBUG_SKIP_CHECK_WIFI {
                
                self.view.makeToast("ไม่พบสัญญาน WiFi กำลังลองอีกครั้ง", duration: 3.0, position: .center)
                var countTry = 0;
                wifiTryTimer = Timer.scheduledTimer(withTimeInterval: 3.0, repeats: true) { (timer) in
                    self.updateWiFiInformation()
                    countTry += 1
                    
                    // found bssid
                    if self.isReceiveWifiBSSID {
                        self.verifyServerAvailable()
                        self.stopUpdateWifiTimer()
                    }
                    // not found bssid
                    else if countTry > 5 {
                        self.stopUpdateWifiTimer()

                        let returnMessage = "ไม่พบสัญญาน WiFi กรุณาตรวจสอบอีกครั้งว่าทำการเชื่อมต่อ WiFi แล้่ว หากยังไม่สำเร็จ กรุณาลอง Restart เครื่องแล้วเข้าใช้งานอีกครั้งค่ะ"
                        Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: returnMessage, isCloseApplication: true)
                    }
                }
            }
            // case found bssid, start action
            else {
              verifyServerAvailable()
            }
            DataFactory.IS_LOADING_FIRSTTIME = false
        }
    }
    
    /**
     part: connect API
     */
 
    // step 1
    func verifyServerAvailable (){
        // data
        // nonokyo: set here
        let data = "HelloServer"
        
        // url
        // nonokyo: set here
        let url = DataFactory.REQUEST_API_SERVER_STATUS_URL
        
        let request:NSMutableURLRequest = NSMutableURLRequest(url: URL(string: url)!)
        request.httpMethod = "POST"
        request.addValue("multipart/form-data; boundary=X", forHTTPHeaderField: "Content-Type")
        
        // post data
        // nonokyo: set here
        let postData = DataFactory.generatePostData(
            [DataFactory.GeneratePostObject(name: "data", value: EncryptDecryptAlgorithm.EncryptData(data)),
             DataFactory.GeneratePostObject(name: "k", value: EncryptDecryptAlgorithm.getKeyEncrypt())]
        )
        // display sent data
        if DataFactory.DEBUG {
            print("url=\(request.url!)")
            print("data=\n\(postData)")
        }
        request.httpBody = postData.data(using: String.Encoding.utf8)
        
        let session = URLSession.shared
        let task = session.dataTask(with: request as URLRequest) { (data, response, error) -> Void in
            if error != nil {
                DispatchQueue.main.async(execute: { () -> Void in
                    Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: DataString.error_connection_timeout, isCloseApplication: true)
                })
                return
            }
            var returnResult = 0
            var returnMessage = ""
            let result = NSString(data: data!, encoding: String.Encoding.utf8.rawValue)! as String
            
            if !result.isEmpty {
                // ========== RESPONSE UNKNOWN / DATA CORRUPTED ===========
                if result.hasPrefix("<!DOCTYPE") {
                    returnMessage = "Error: return data corrupted, please contact administrator."
                    returnResult = 0
                }
                    // ========== RESPONSE ERROR =============
                else if result == "IamNotAndroid" {
                    returnMessage = "กรุณาเชื่อมต่อ Wifi ภายในสำนักงาน"
                    returnResult = 0
                }
                    // ========== RESPONSE SUCCESS ===========
                else if result == "HiAndroid" {
                    returnResult = 1
                }
                    // ========== RESPONSE UNKNOWN / DATA CORRUPTED ===========
                else {
                    returnMessage = "Error: return data corrupted, please contact administrator. message: " + result
                    returnResult = 0
                }
            }
            else {
                returnMessage = "response == null"
                returnResult = 0
            }
            
            if returnResult == 1 {
                
                self.submitNetworkLog()
            }
            else {
                Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: returnMessage, isCloseApplication: true)
            }
        }
        task.resume()
    }
    
    // step 2
    func submitNetworkLog (){
        // data
        // nonokyo: set here
        let macAdr = appDelegate.UUID
        let publicIP = appDelegate.publicIP
        let privateIP = appDelegate.privateIP
        
        
        // url
        // nonokyo: set here
        let url = DataFactory.SUBMIT_API_LOG_IP_URL
        
        let request:NSMutableURLRequest = NSMutableURLRequest(url: URL(string: url)!)
        request.httpMethod = "POST"
        request.addValue("multipart/form-data; boundary=X", forHTTPHeaderField: "Content-Type")
        
        // post data
        // nonokyo: set here
        let postData = DataFactory.generatePostData(
            [DataFactory.GeneratePostObject(name: "MacData", value: EncryptDecryptAlgorithm.EncryptData(macAdr)),
             DataFactory.GeneratePostObject(name: "Mack", value: EncryptDecryptAlgorithm.getKeyEncrypt()),
             DataFactory.GeneratePostObject(name: "PubIPData", value: EncryptDecryptAlgorithm.EncryptData(publicIP)),
             DataFactory.GeneratePostObject(name: "PubIPk", value: EncryptDecryptAlgorithm.getKeyEncrypt()),
             DataFactory.GeneratePostObject(name: "PrivateIPData", value: EncryptDecryptAlgorithm.EncryptData(privateIP)),
             DataFactory.GeneratePostObject(name: "PrivateIPk", value: EncryptDecryptAlgorithm.getKeyEncrypt())]
        )
        // display sent data
        if DataFactory.DEBUG {
            print("url=\(request.url!)")
            print("data=\n\(postData)")
        }
        request.httpBody = postData.data(using: String.Encoding.utf8)
        
        /*
         DispatchQueue.main.async(execute: { () -> Void in
         self.loadingProgress.startAnimating()
         })
         */
        
        let session = URLSession.shared
        let task = session.dataTask(with: request as URLRequest) { (data, response, error) -> Void in
            if error != nil {
                DispatchQueue.main.async(execute: { () -> Void in
                    /*
                     self.loadingProgress.stopAnimating()
                     */
                    Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: DataString.error_connection_timeout, isCloseApplication: true)
                })
                return
            }
            var returnResult = 0
            var returnMessage = ""
            let result = NSString(data: data!, encoding: String.Encoding.utf8.rawValue)! as String
            
            if !result.isEmpty {
                // ========== RESPONSE UNKNOWN / DATA CORRUPTED ===========
                if result.hasPrefix("<!DOCTYPE") {
                    returnMessage = "Error: return data corrupted, please contact administrator."
                    returnResult = 0
                }
                    // ========== RESPONSE ERROR =============
                else if result == "IPRecordCompleteButHack" {
                    returnMessage = "!!!Abnormal Connection are Detected!!!"
                    returnResult = 0
                }
                    // ========== RESPONSE SUCCESS ===========
                else if result == "IPRecordComplete" {
                    returnResult = 1
                }
                    // ========== RESPONSE UNKNOWN / DATA CORRUPTED ===========
                else {
                    returnMessage = "Error: return data corrupted, please contact administrator. message: " + result
                    returnResult = 0
                }
            }
            else {
                returnMessage = "response == null"
                returnResult = 0
            }
            
            if returnResult == 1 {
                self.submitBSSID()
            }
            else {
                Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: returnMessage, isCloseApplication: true)
            }
        }
        task.resume()
    }
    
    // step 3
    /**
     NOTE: only this step in DEBUG MODE, that need to disable check bssid for now
     */
    func submitBSSID (){
        // verify BSSID
        
        // ptoon: for test only, skip submit bssid
        // must remove
        /*
        if DataFactory.DEBUG {
            self.requestAuthenticationFromMACAddress()
            return
        }
        */
        
        // data
        // nonokyo: set here
        let macAdr = appDelegate.UUID
        let bssid = appDelegate.BSSID
        let ssid = appDelegate.SSID
        
        // url
        // nonokyo: set here
        let url = DataFactory.REQUEST_API_CHECK_INDOOR_WIFI_URL
        
        let request:NSMutableURLRequest = NSMutableURLRequest(url: URL(string: url)!)
        request.httpMethod = "POST"
        request.addValue("multipart/form-data; boundary=X", forHTTPHeaderField: "Content-Type")
        
        // post data
        // nonokyo: set here
        let postData = DataFactory.generatePostData(
            [DataFactory.GeneratePostObject(name: "WifiBSSIDData", value: EncryptDecryptAlgorithm.EncryptData(bssid)),
             DataFactory.GeneratePostObject(name: "WifiBSSIDk", value: EncryptDecryptAlgorithm.getKeyEncrypt()),
             DataFactory.GeneratePostObject(name: "WifiRssid_Data", value: EncryptDecryptAlgorithm.EncryptData(ssid)),
             DataFactory.GeneratePostObject(name: "WifiRssid_k", value: EncryptDecryptAlgorithm.getKeyEncrypt()),
             DataFactory.GeneratePostObject(name: "MobileMacData", value: EncryptDecryptAlgorithm.EncryptData(macAdr)),
             DataFactory.GeneratePostObject(name: "MobileMack", value: EncryptDecryptAlgorithm.getKeyEncrypt())
            ]
        )
        // display sent data
        if DataFactory.DEBUG {
            print("url=\(request.url!)")
            print("data=\n\(postData)")
        }
        request.httpBody = postData.data(using: String.Encoding.utf8)
        
        /*
         DispatchQueue.main.async(execute: { () -> Void in
         self.loadingProgress.startAnimating()
         })
         */
        
        let session = URLSession.shared
        let task = session.dataTask(with: request as URLRequest) { (data, response, error) -> Void in
            if error != nil {
                DispatchQueue.main.async(execute: { () -> Void in
                    /*
                     self.loadingProgress.stopAnimating()
                     */
                    Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: DataString.error_connection_timeout, isCloseApplication: true)
                })
                return
            }
            var returnResult = 0
            var returnMessage = ""
            let result = NSString(data: data!, encoding: String.Encoding.utf8.rawValue)! as String
            
            if !result.isEmpty {
                // ========== RESPONSE UNKNOWN / DATA CORRUPTED ===========
                if result.hasPrefix("<!DOCTYPE") {
                    returnMessage = "Error: return data corrupted, please contact administrator."
                    returnResult = 0
                    DataFactory.IS_CHECKIN_ALLOW = 0
                }
                    // ========== RESPONSE ERROR =============
                else if result == "Notallow" {
                    returnMessage = "กรุณาลงเวลาในพื้นที่ที่คุณมีสิทธิ์"
                    returnResult = 0
                    DataFactory.IS_CHECKIN_ALLOW = 0
                }
                    // ========== RESPONSE SUCCESS ===========
                else if result == "Allow" {
                    returnResult = 1
                    DataFactory.IS_CHECKIN_ALLOW = 1
                }
                    // ========== RESPONSE UNKNOWN / DATA CORRUPTED ===========
                else {
                    returnMessage = "Error: return data corrupted, please contact administrator. message: " + result
                    returnResult = 0
                    DataFactory.IS_CHECKIN_ALLOW = 0
                }
            }
            else {
                returnMessage = "response == null"
                returnResult = 0
            }
            
            if DataFactory.IS_CHECKIN_ALLOW >= 0 {
               self.requestAuthenticationFromMACAddress()
            }
            else {
                Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: returnMessage, isCloseApplication: true)
            }
            
            /*if returnResult == 1 {
                self.requestAuthenticationFromMACAddress()
            }
            else {
                Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: returnMessage, isCloseApplication: true)
            }*/
        }
        task.resume()
        
        //requestAuthenticationFromMACAddress()
    }
    
    // step 4
   
    // status = 5
    func requestAuthenticationFromMACAddress (){
        // verify MAC Address
        
        // data
        // nonokyo: set here
        let status = 5
        let macAdr = appDelegate.UUID
        let data = "\(status)+\(macAdr)"
        
        // url
        // nonokyo: set here
        let url = DataFactory.REQUEST_API_CHECK_MACADDRESS_URL
        
        let request:NSMutableURLRequest = NSMutableURLRequest(url: URL(string: url)!)
        request.httpMethod = "POST"
        request.addValue("multipart/form-data; boundary=X", forHTTPHeaderField: "Content-Type")
        
        // post data
        // nonokyo: set here
        let postData = DataFactory.generatePostData(
            [DataFactory.GeneratePostObject(name: "data_both", value: EncryptDecryptAlgorithm.EncryptData(data)),
             DataFactory.GeneratePostObject(name: "data_bothK", value: EncryptDecryptAlgorithm.getKeyEncrypt())]
        )
        // display sent data
        if DataFactory.DEBUG {
            print("url=\(request.url!)")
            print("data=\n\(postData)")
        }
        request.httpBody = postData.data(using: String.Encoding.utf8)
        
        /*
         DispatchQueue.main.async(execute: { () -> Void in
         self.loadingProgress.startAnimating()
         })
         */
        
        let session = URLSession.shared
        let task = session.dataTask(with: request as URLRequest) { (data, response, error) -> Void in
            if error != nil {
                DispatchQueue.main.async(execute: { () -> Void in
                    /*
                     self.loadingProgress.stopAnimating()
                     */
                    Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: DataString.error_connection_timeout, isCloseApplication: true)
                })
                return
            }
            var returnResult = 0
            var returnMessage = ""
            let result = NSString(data: data!, encoding: String.Encoding.utf8.rawValue)! as String
            
            if !result.isEmpty {
                // ========== RESPONSE UNKNOWN / DATA CORRUPTED ===========
                if result.hasPrefix("<!DOCTYPE") {
                    returnMessage = "Error: return data corrupted, please contact administrator."
                    returnResult = 0
                }
                    // ========== RESPONSE SUCCESS ===========
                else if result == "LoginStatusZero" {
                    // go to 3.
                    returnResult = 1
                }
                else if result == "NoMacAddress" {
                    // go to 2.
                    returnResult = 2
                }
                else if result == "LoginStatusOne" {
                    // go to 1.8.
                    returnResult = 3
                }
                    // ========== RESPONSE UNKNOWN / DATA CORRUPTED ===========
                else {
                    returnMessage = "Error: return data corrupted, please contact administrator. message: " + result
                    returnResult = 0
                }
            }
            else {
                returnMessage = "response == null"
                returnResult = 0
            }
            
            // login
            if returnResult == 1 {
                self.gotoSigninPage()
            }
                // register
            else if returnResult == 2 {
                self.gotoRegisterPage()
            }
                // checkin
            else if returnResult == 3 {
                self.gotoCheckinPage()
            }
            else {
                Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: returnMessage, isCloseApplication: true)
            }
        }
        task.resume()
    }

    /**
     part: etc
     */
    func gotoRegisterPage() {
        DispatchQueue.main.async(execute: { () -> Void in
            self.performSegue(withIdentifier: "segue_loading_to_register1", sender: nil)
        })
    }
    
    func gotoSigninPage() {
        DispatchQueue.main.async(execute: { () -> Void in
            self.performSegue(withIdentifier: "segue_loading_to_signin", sender: nil)
        })
    }
    
    func gotoCheckinPage() {
        DispatchQueue.main.async(execute: { () -> Void in
            self.performSegue(withIdentifier: "segue_loading_to_checkin", sender: nil)
        })
    }
    
    func locationAuthorizationStatus() -> CLAuthorizationStatus {
        var locationAuthorizationStatus : CLAuthorizationStatus
        if #available(iOS 14.0, *) {
            locationAuthorizationStatus =  locationManager.authorizationStatus
        } else {
            // Fallback on earlier versions
            locationAuthorizationStatus = CLLocationManager.authorizationStatus()
        }
        return locationAuthorizationStatus
    }
    
    /**
     part: location manager service
     */
    func locationManager(_ manager: CLLocationManager, didChangeAuthorization status: CLAuthorizationStatus) {
        
        if status == .authorizedWhenInUse {
            updateWiFiInformation()
        }
    }
    
    func updateWiFiInformation() {
        // note: if bssid is null, try restart device first
        print("SSID: \(currentNetworkInfos?.first?.ssid ?? "")")
        print("BSSID: \(currentNetworkInfos?.first?.bssid ?? "")")
        
        if let bssid = currentNetworkInfos?.first?.bssid {
            appDelegate.SSID = currentNetworkInfos?.first?.ssid ?? ""
            appDelegate.BSSID = verifyBSSID(bssid: bssid) ?? ""
            DataFactory.CURRENT_BSSID = appDelegate.BSSID
            print("SSID: \(appDelegate.SSID)")
            print("BSSID: \(appDelegate.BSSID)")
            isReceiveWifiBSSID = true
        }
        else {
            print("if bssid is null, try restart device first")
        }
    }
    
    func stopUpdateWifiTimer() {
        if wifiTryTimer != nil {
            wifiTryTimer!.invalidate()
            wifiTryTimer = nil
        }
    }
    
    func verifyBSSID(bssid: String?) -> String? {
        if bssid == nil {
            return nil
        }
        let originBSSIDArray = bssid!.split(separator: ":")
        var newBSSID = ""
        
        if originBSSIDArray.count == 6 {
            let bssid0 = originBSSIDArray[0]
            let bssid1 = originBSSIDArray[1]
            let bssid2 = originBSSIDArray[2]
            let bssid3 = originBSSIDArray[3]
            let bssid4 = originBSSIDArray[4]
            let bssid5 = originBSSIDArray[5]
            
            if bssid0.count == 1 { newBSSID = "0\(bssid0)" }
            else { newBSSID = String(bssid0) }
            
            if bssid1.count == 1 { newBSSID = "\(newBSSID):0\(bssid1)" }
            else { newBSSID = "\(newBSSID):\(bssid1)" }
            
            if bssid2.count == 1 { newBSSID = "\(newBSSID):0\(bssid2)" }
            else { newBSSID = "\(newBSSID):\(bssid2)" }
            
            if bssid3.count == 1 { newBSSID = "\(newBSSID):0\(bssid3)" }
            else { newBSSID = "\(newBSSID):\(bssid3)" }
            
            if bssid4.count == 1 { newBSSID = "\(newBSSID):0\(bssid4)" }
            else { newBSSID = "\(newBSSID):\(bssid4)" }
            
            if bssid5.count == 1 { newBSSID = "\(newBSSID):0\(bssid5)" }
            else { newBSSID = "\(newBSSID):\(bssid5)" }
        }
        else {
            return nil
        }
        
        return newBSSID
    }
    
    /**
     part: destroy
     */
    
    deinit {
        stopUpdateWifiTimer()
    }
    
    
   
    
    
}
