//
//  Register1Controller.swift
//  TIME
//
//  Created by Sruit Angkavanitsuk on 2/7/20.
//  Copyright © 2020 NSTDA. All rights reserved.
//

import UIKit

class Register1Controller: UIViewController {
    
    @IBOutlet weak var ivLogo: UIImageView!
    
    var appDelegate:AppDelegate!
    
    fileprivate var loadingProgress : UIActivityIndicatorView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        appDelegate = UIApplication.shared.delegate as! AppDelegate
        if #available(iOS 13.0, *) {
            overrideUserInterfaceStyle = .light // force light mode
        }
        
        // loading view
        if DataFactory.BUSINESS_CODE == 1{
            ivLogo.image = UIImage(named: "i_nectec_main_logo1")
        }
        else if DataFactory.BUSINESS_CODE == 2 {
            ivLogo.image = UIImage(named: "i_nbtc_main_logo1")
        }
        
        // loading animation
        loadingProgress = UIActivityIndicatorView() as UIActivityIndicatorView
        loadingProgress.frame = CGRect(x: 0, y: 0,
            width: UIScreen.main.bounds.size.width,
            height: UIScreen.main.bounds.size.height)
        loadingProgress.isOpaque = false
        loadingProgress.backgroundColor = UIColor( red: 0.95, green: 0.95, blue: 0.95, alpha: 0.5 )
        loadingProgress.hidesWhenStopped = true
        loadingProgress.style = UIActivityIndicatorView.Style.gray
        view.addSubview(loadingProgress)
        
        //show toast
        self.view.makeToast("กรุณาลงทะเบียน" , duration: 1.3, position: .bottom)
        //self.view.makeToast("กรุณาลงทะเบียน", point: CGPoint(x: 110.0, y: 110.0)); title: nil, image: nil)
    }
    
    func requestCheckAccountEnroll() {
        
        // data
        // ptoon: set here
        let data = appDelegate.UUID
        
        // url
        // ptoon: set here
        let url = DataFactory.REQUEST_API_CHECK_ACCOUNT_ENROLL_URL
        
        let request:NSMutableURLRequest = NSMutableURLRequest(url: URL(string: url)!)
        request.httpMethod = "POST"
        request.addValue("multipart/form-data; boundary=X", forHTTPHeaderField: "Content-Type")
        
        // post data
        // ptoon: set here
        let postData = DataFactory.generatePostData(
            [DataFactory.GeneratePostObject(name: "data_Mac", value: EncryptDecryptAlgorithm.EncryptData(data)),
             DataFactory.GeneratePostObject(name: "data_MacK", value: EncryptDecryptAlgorithm.getKeyEncrypt())]
        )
        // display sent data
        if DataFactory.DEBUG {
            print("url=\(request.url!)")
            print("data=\n\(postData)")
        }
        request.httpBody = postData.data(using: String.Encoding.utf8)
        
        DispatchQueue.main.async(execute: { () -> Void in
            self.loadingProgress.startAnimating()
        })
        
        let session = URLSession.shared
        let task = session.dataTask(with: request as URLRequest) { (data, response, error) -> Void in
            
            if error != nil {
                DispatchQueue.main.async(execute: { () -> Void in
                    self.loadingProgress.stopAnimating()
                    Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: DataString.error_connection_timeout)
                })
                return
            }
            
            var returnResult = 0
            var returnMessage = ""
            let result = NSString(data: data!, encoding: String.Encoding.utf8.rawValue)! as String
            
            if !result.isEmpty {
                // ========== RESPONSE UNKNOWN / DATA CORRUPTED ===========
                if result.hasPrefix("<!DOCTYPE") {
                    returnMessage = "Error: return data corrupted, please contact administrator."
                    returnResult = 0
                }
                // ========== RESPONSE ERROR =============
                else if result == "AlreadyAccount" {
                    returnMessage = "มีผู้ใช้งานบัญชีอื่นในอุปกรณ์นี้แล้ว"
                    returnResult = 0
                }
                // ========== RESPONSE SUCCESS ===========
                else if result == "NoAccount" {
                    returnResult = 1
                }
                // ========== RESPONSE UNKNOWN / DATA CORRUPTED ===========
                else {
                    returnMessage = "Error: return data corrupted, please contact administrator. message: " + result
                    returnResult = 0
                }
            }
            else {
                returnMessage = "response == null"
                returnResult = 0
            }
            
            if returnResult == 1 {
                DispatchQueue.main.async(execute: { () -> Void in
                    self.loadingProgress.stopAnimating()
                    // set segue to new page here
                    // ptoon: set here
                    self.performSegue(withIdentifier: "segue_register1_to_register2", sender: nil)
                })
            }
            else {
                DispatchQueue.main.async(execute: { () -> Void in
                    self.loadingProgress.stopAnimating()
                    Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: returnMessage)
                })
            }
        }
        task.resume()
    }
    
    @IBAction func bRegisterClicked(_ sender: UIButton) {
        // start
        requestCheckAccountEnroll()
    }
    
    @IBAction func bCloseClicked(_ sender: UIButton) {
    }
    
    @IBAction func segue_exit_to_register1(_ segue: UIStoryboardSegue) {
    }
    
    @IBAction func bFAQClicked(_ sender: UIButton) {
        /*var discTxt = "วัตถุประสงค์นี้ใช้เพื่อสำหรับการลงเวลาด้วยวิธีการตรวจสอบปัจจัยที่หลากหลายได้แก่ ใบหน้า รหัสผ่าน โทรศัพท์มือถือและสถานที่ ระบบนี้จะมีการบันทึกข้อมูลใบหน้าของผู้ใช้งาน และข้อมูลใบหน้าอาจถูกนำมาใช้เพื่อปรับปรุงระบบให้ดียิ่งขึ้นต่อไปในอนาคตและจะไม่มีการนำข้อมูลใบหน้าของผู้ใช้งานไปใช้ในวัตถุประสงค์อื่นนอกเหนือจากงานวิจัยภานใน ปัจจุบันระบบนี้ยังอยู่ระหว่างทดลองการใช้งาน"
        
        let alert = UIAlertController(title: "Disclamer", message: discTxt, preferredStyle: .alert)
        alert.addAction(UIAlertAction(title: "รับทราบ", style: .cancel, handler: nil))

        self.present(alert, animated: true)*/
        let urlComponents = URLComponents (string: "http://attime.secteam.in.th/#faq")!
        if #available(iOS 10.0, *) {
            UIApplication.shared.open(urlComponents.url!, options: [:], completionHandler: nil)
        } else {
            UIApplication.shared.openURL(urlComponents.url!)
        }
    }
    
    /**
    * Called when the user click on the view (outside the UITextField).
    */
    override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?) {
        self.view.endEditing(true)
    }
    
}

