//
//  Register2Controller.swift
//  TIME
//
//  Created by Sruit Angkavanitsuk on 2/7/20.
//  Copyright © 2020 NSTDA. All rights reserved.
//

import UIKit

class Register2Controller: UIViewController {

    @IBOutlet weak var ivLogo: UIImageView!
    @IBOutlet weak var lText: UILabel!
    @IBOutlet weak var tfUsername: UITextField!
    @IBOutlet weak var tfPassword: UITextField!
    fileprivate var loadingProgress : UIActivityIndicatorView!
    private var forthData = ""
    
    var appDelegate:AppDelegate!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        appDelegate = UIApplication.shared.delegate as! AppDelegate
        if #available(iOS 13.0, *) {
            overrideUserInterfaceStyle = .light // force light mode
        }
        // Do any additional setup after loading the view.
        
        
        
        // loading view
        if DataFactory.BUSINESS_CODE == 1 {
            ivLogo.image = UIImage(named: "i_nectec_main_logo2")
            lText.text = "กรอกข้อมูลผู้ใช้งาน\nลงทะเบียนโดยใช้บัญชีพนักงานสวทช.\nในการสร้างผู้ใช้งานใหม่"
        }
        else if DataFactory.BUSINESS_CODE == 2 {
            ivLogo.image = UIImage(named: "i_nbtc_main_logo2")
            lText.text = "กรอกข้อมูลผู้ใช้งาน\nลงทะเบียนโดยใช้บัญชีพนักงานกสทช.\nในการสร้างผู้ใช้งานใหม่"
        }
        
        // loading animation
        loadingProgress = UIActivityIndicatorView() as UIActivityIndicatorView
        loadingProgress.frame = CGRect(x: 0, y: 0,
            width: UIScreen.main.bounds.size.width,
            height: UIScreen.main.bounds.size.height)
        loadingProgress.isOpaque = false
        loadingProgress.backgroundColor = UIColor( red: 0.95, green: 0.95, blue: 0.95, alpha: 0.5 )
        loadingProgress.hidesWhenStopped = true
        loadingProgress.style = UIActivityIndicatorView.Style.gray
        view.addSubview(loadingProgress)
    }
    
    override func viewWillAppear(_ animated: Bool) {
        NotificationCenter.default.addObserver(self, selector: #selector(keyboardWillShow), name: UIResponder.keyboardWillShowNotification, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(keyboardWillHide), name: UIResponder.keyboardWillHideNotification, object: nil)
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        NotificationCenter.default.removeObserver(self, name: UIResponder.keyboardWillShowNotification, object: self.view.window)
        NotificationCenter.default.removeObserver(self, name: UIResponder.keyboardWillHideNotification, object: self.view.window)
    }
    
    func submitRegisterAccount(){
        
        // data
        // nonokyo: set here
        let username = tfUsername.text!;
        let password = tfPassword.text!;
        
        if username.isEmpty {
            //Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: "กรุณากรอกชื่อบัญชีค่ะ")
            self.view.makeToast("กรุณากรอกชื่อบัญชี", duration: 1.3, position: .bottom)
            return
        }
        else if password.isEmpty {
            //Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: "กรุณากรอกรหัสผ่านค่ะ")
            self.view.makeToast("กรุณากรอกรหัสผ่าน", duration: 1.3, position: .bottom)
            return
        }
        else if password.length < 6 {
            //Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: "รหัสผ่านต้องมีความยาวอย่างน้อย 6 ตัวอักษรค่ะ")
             self.view.makeToast("ไม่สามารถลงทะเบียนได้", duration: 1.3, position: .bottom)
            return
        }
        
        let status = "7"; // no account, ready to create new account
        var business = ""
        if DataFactory.BUSINESS_CODE == 1 {
            business = "NECTEC"
        }
        else if DataFactory.BUSINESS_CODE == 2 {
            business = "NBTC"
        }
        forthData = "\(status)+\(username)+\(password)+\(business)"

        if DataFactory.DEBUG {
            print("dataforth=\(forthData)")
            print("password=\(password)")
        }
        // url
        // nonokyo: set here
        let url = DataFactory.SUBMIT_API_ACCOUNT_REGISTER_ENROLL_LDAP_URL
        
        let request:NSMutableURLRequest = NSMutableURLRequest(url: URL(string: url)!)
        request.httpMethod = "POST"
        request.addValue("multipart/form-data; boundary=X", forHTTPHeaderField: "Content-Type")
        
        // post data
        // nonokyo: set here
        let postData = DataFactory.generatePostData(
            [DataFactory.GeneratePostObject(name: "data_forth", value: EncryptDecryptAlgorithm.EncryptData(forthData)),
             DataFactory.GeneratePostObject(name: "data_forthK", value: EncryptDecryptAlgorithm.getKeyEncrypt()),
             DataFactory.GeneratePostObject(name: "data_passwordEnroll",value:EncryptDecryptAlgorithm.EncryptData(password)),
             DataFactory.GeneratePostObject(name: "data_passwordEnrollK", value: EncryptDecryptAlgorithm.getKeyEncrypt())]
        )
        // display sent data
        if DataFactory.DEBUG {
            print("url=\(request.url!)")
            print("data=\n\(postData)")
        }
        request.httpBody = postData.data(using: String.Encoding.utf8)
        
        DispatchQueue.main.async(execute: { () -> Void in
            self.loadingProgress.startAnimating()
        })
        
        let session = URLSession.shared
        let task = session.dataTask(with: request as URLRequest) { (data, response, error) -> Void in
            
            if error != nil {
                DispatchQueue.main.async(execute: { () -> Void in
                    self.loadingProgress.stopAnimating()
                    Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: DataString.error_connection_timeout)
                })
                return
            }
            
            var returnResult = 0
            var returnMessage = ""
            let result = NSString(data: data!, encoding: String.Encoding.utf8.rawValue)! as String
            if DataFactory.DEBUG {
                print("result=\(result)")
            }
            
            if !result.isEmpty {
                // ========== RESPONSE UNKNOWN / DATA CORRUPTED ===========
                if result.hasPrefix("<!DOCTYPE") {
                    returnMessage = "Error: return data corrupted, please contact administrator."
                    returnResult = 0
                }
                // ========== RESPONSE ERROR =============
                else if result == "PasswordBlank"{
                    returnMessage = "กรุณากรอกรหัสผ่าน"
                    returnResult = 0
                }
                else if result == "authenticationErrorNoAccount" {
                    //returnMessage = "ชื่อบัญชีหรือรหัสผ่านไม่ถูกต้อง กรุณาลองใหม่อีกครั้ง"
                    returnMessage = "ไม่สามารถลงทะเบียนได้"
                    returnResult = 0
                }
                else if result == "authenticationErrorAlreadyAccount" {
                    returnMessage = "มีผู้ใช้งานบัญชีนี้ในอุปกรณ์อื่นแล้ว"
                    returnResult = 0
                }
                else if result == "authenticationPassAlreadyAccount"{
                    returnMessage = "มีผู้ใช้งานบัญชีนี้ในอุปกรณ์อื่นแล้ว หรือหากต้องการลบชื่อบัญชีนี้ในอุปกรณ์อื่นให้ติดต่อเจ้าหน้าที่ ท่านสามารถอ่านข้อมูลเพิ่มเติมได้ที่ FAQ"
                    returnResult = 0
                }
                // ========== RESPONSE SUCCESS ===========
                else if result == "authenticationPassNoAccount" {
                    returnResult = 1
                }
                // ========== RESPONSE UNKNOWN / DATA CORRUPTED ===========
                else {
                    returnMessage = "Error: return data corrupted, please contact administrator. message: " + result
                    returnResult = 0
                }
            }
            else {
                returnMessage = "response == null"
                returnResult = 0
            }
            
            if returnResult == 1 {
                DispatchQueue.main.async(execute: { () -> Void in
                    self.loadingProgress.stopAnimating()
                    // set segue to new page here
                    self.tfPassword.text = ""
                    // nonokyo: set here
                    self.performSegue(withIdentifier: "segue_register2_to_captprepare", sender: nil)
                })
            }
            else {
               if returnMessage.hasPrefix("Error:"){
                    DispatchQueue.main.async(execute: { () -> Void in
                        self.loadingProgress.stopAnimating()
                        Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: returnMessage)
                    })
                }
                else {
                    DispatchQueue.main.async(execute: { () -> Void in
                        self.loadingProgress.stopAnimating()
                        self.view.makeToast(returnMessage, duration: 1.3, position: .bottom)
                    })
                }
            }
        }
        task.resume()
    }
    
    @IBAction func bRegisterClicked(_ sender: UIButton) {
        
        // test
        let dataEnc = EncryptDecryptAlgorithm.EncryptData("HelloServer")
        print("result=\(dataEnc)")
        
        //start
        submitRegisterAccount()
    }
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "segue_register2_to_captprepare" {
            let capturePrepareController = segue.destination as! CapturePrepareController
            capturePrepareController.forthData = forthData
            capturePrepareController.from_code = "REGISTER"
        }
    }
    
    @IBAction func bCloseClicked(_ sender: UIButton) {
        self.performSegue(withIdentifier: "segue_exit_to_register1", sender: nil)
    }
    
    @IBAction func segue_exit_to_register2(_ segue: UIStoryboardSegue) {
    }
    
    /**
    * Called when the user click on the view (outside the UITextField).
    */
    override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?) {
        self.view.endEditing(true)
    }

    @objc func keyboardWillShow(notification: NSNotification) {
        if let keyboardSize = (notification.userInfo?[UIResponder.keyboardFrameEndUserInfoKey] as? NSValue)?.cgRectValue {
            if self.view.frame.origin.y == 0 {
                self.view.frame.origin.y -= keyboardSize.height
            }
        }
    }

    @objc func keyboardWillHide(notification: NSNotification) {
        if self.view.frame.origin.y != 0 {
            self.view.frame.origin.y = 0
        }
    }
}

