//
//  RemoveController.swift
//  TIME
//
//  Created by Sruit Angkavanitsuk on 3/7/20.
//  Copyright © 2020 NSTDA. All rights reserved.
//

import UIKit

class RemoveController: UIViewController {
    
    var stepRemove = 1
    var empID = ""
    private var threeData = ""
    
    @IBOutlet weak var ivLogo: UIImageView!
    @IBOutlet weak var vStep1: UIView!
    @IBOutlet weak var vStep2: UIView!
    @IBOutlet weak var lUsername: UILabel!
    @IBOutlet weak var lUsernameDisplay: UILabel!
    @IBOutlet weak var tfPassword: UITextField!
    @IBOutlet weak var bRemove: UILabel!
    
    var appDelegate:AppDelegate!
    var idleCheck:DispatchWorkItem!
    
    fileprivate var loadingProgress : UIActivityIndicatorView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        appDelegate = UIApplication.shared.delegate as! AppDelegate
        if #available(iOS 13.0, *) {
            overrideUserInterfaceStyle = .light // force light mode
        }
        // Do any additional setup after loading the view.
        
        // loading view
        if DataFactory.BUSINESS_CODE == 1{
            ivLogo.image = UIImage(named: "i_nectec_remove_logo")
        }
        else if DataFactory.BUSINESS_CODE == 2 {
            ivLogo.image = UIImage(named: "i_nbtc_remove_logo")
        }
        
        vStep1.isHidden = false
        vStep2.isHidden = true
        bRemove.text = "LOGIN"
        lUsername.text = empID
        lUsernameDisplay.text = empID
        
        idleCheck = DispatchWorkItem(block: {
            // if stay in this page longer than 300 seconds
            // refresh in 300 sec
            DataFactory.IS_LOADING_FIRSTTIME = true
            self.performSegue(withIdentifier: "segue_exit_remove_to_loading", sender: nil)
        })
        
        // loading animation
        loadingProgress = UIActivityIndicatorView() as UIActivityIndicatorView
        loadingProgress.frame = CGRect(x: 0, y: 0,
                                       width: UIScreen.main.bounds.size.width,
                                       height: UIScreen.main.bounds.size.height)
        loadingProgress.isOpaque = false
        loadingProgress.backgroundColor = UIColor( red: 0.95, green: 0.95, blue: 0.95, alpha: 0.5 )
        loadingProgress.hidesWhenStopped = true
        loadingProgress.style = UIActivityIndicatorView.Style.gray
        view.addSubview(loadingProgress)
    }
    
    override func viewWillAppear(_ animated: Bool) {
        NotificationCenter.default.addObserver(self, selector: #selector(keyboardWillShow), name: UIResponder.keyboardWillShowNotification, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(keyboardWillHide), name: UIResponder.keyboardWillHideNotification, object: nil)
        
        DispatchQueue.main.asyncAfter(deadline: .now() + 300, execute: idleCheck)
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        NotificationCenter.default.removeObserver(self, name: UIResponder.keyboardWillShowNotification, object: self.view.window)
        NotificationCenter.default.removeObserver(self, name: UIResponder.keyboardWillHideNotification, object: self.view.window)
        
        idleCheck.cancel()
    }
    
    /**
     part: connect API
     */
    
    // status = 10
    func submitConfirmPassword (){
        
        var password = tfPassword.text!;
        let status = "10";
        var business = ""
        
        if password.isEmpty {
            //Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: "กรุณากรอกรหัสผ่านค่ะ")
            self.view.makeToast("กรุณากรอกรหัสผ่าน", duration: 1.3, position: .bottom)
            return
        }
        else if password.length < 6 {
            //Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: "รหัสผ่านต้องมีความยาวอย่างน้อย 6 ตัวอักษรค่ะ")
            self.view.makeToast("ล็อกอินไม่สำเร็จ กรุณาลองใหม่อีกครั้ง", duration: 1.3, position: .bottom)
            return
        }
        
        // data
        // nonokyo: set here
        if DataFactory.BUSINESS_CODE == 1 {
            business = "NECTEC"
        }
        else if DataFactory.BUSINESS_CODE == 2 {
            business = "NBTC"
        }
        threeData = "\(status)+\(empID)+\(password)+\(business)"
        if DataFactory.DEBUG {
            print("threeData=\(threeData)")
        }
        
        // url
        // nonokyo: set here
        let url = DataFactory.SUBMIT_API_ACCOUNT_DELETE_VERIFY_PASSWORD_URL
        
        let request:NSMutableURLRequest = NSMutableURLRequest(url: URL(string: url)!)
        request.httpMethod = "POST"
        request.addValue("multipart/form-data; boundary=X", forHTTPHeaderField: "Content-Type")
        
        // post data
        // nonokyo: set here
        let postData = DataFactory.generatePostData(
            [DataFactory.GeneratePostObject(name: "three_data", value: EncryptDecryptAlgorithm.EncryptData(threeData)),
             DataFactory.GeneratePostObject(name: "threeK", value: EncryptDecryptAlgorithm.getKeyEncrypt()),
             DataFactory.GeneratePostObject(name: "passwordDelete_data",value:EncryptDecryptAlgorithm.EncryptData(password)),
             DataFactory.GeneratePostObject(name: "passwordDeleteK", value: EncryptDecryptAlgorithm.getKeyEncrypt())]
        )
        // display sent data
        if DataFactory.DEBUG {
            print("url=\(request.url!)")
            print("data=\n\(postData)")
        }
        request.httpBody = postData.data(using: String.Encoding.utf8)
        
        DispatchQueue.main.async(execute: { () -> Void in
            self.loadingProgress.startAnimating()
        })
        
        let session = URLSession.shared
        let task = session.dataTask(with: request as URLRequest) { (data, response, error) -> Void in
            
            if error != nil {
                DispatchQueue.main.async(execute: { () -> Void in
                    self.loadingProgress.stopAnimating()
                    Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: DataString.error_connection_timeout)
                })
                return
            }
            
            var returnResult = 0
            var returnMessage = ""
            let result = NSString(data: data!, encoding: String.Encoding.utf8.rawValue)! as String
            
            if !result.isEmpty {
                // ========== RESPONSE UNKNOWN / DATA CORRUPTED ===========
                if result.hasPrefix("<!DOCTYPE") {
                    returnMessage = "Error: return data corrupted, please contact administrator."
                    returnResult = 0
                }
                    // ========== RESPONSE ERROR =============
                else if result == "Login Fail.Try Again" {
                    returnMessage = "ล็อกอินไม่สำเร็จ กรุณาลองใหม่อีกครั้ง"
                    returnResult = 0
                }
                else if result == "PasswordBlank"{
                    returnMessage = "กรุณากรอกรหัสผ่าน"
                    returnResult = 0
                }
                    // ========== RESPONSE SUCCESS ===========
                else {
                    self.empID = result
                    //returnMessage = "พนักงานบัญชีชื่อ " + self.empName + " ล็อกอินสำเร็จ"
                    returnResult = 1
                }
            }
            else {
                returnMessage = "response == null"
                returnResult = 0
            }
            
            if returnResult == 1 {
                DispatchQueue.main.async(execute: { () -> Void in
                    self.loadingProgress.stopAnimating()
                    
                    self.stepRemove = 2;
                    self.vStep1.isHidden = true
                    self.vStep2.isHidden = false
                    self.bRemove.text = "ลบ"
                    
                    //Dialog.showSuccess(self, title: DataString.TITLE_SUCCESS_ALERT, detail: returnMessage)
                })
            }
            else {
                if returnMessage.hasPrefix("Error:"){
                    DispatchQueue.main.async(execute: { () -> Void in
                        self.loadingProgress.stopAnimating()
                        Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: returnMessage)
                    })
                }
                else {
                    DispatchQueue.main.async(execute: { () -> Void in
                        self.loadingProgress.stopAnimating()
                        self.view.makeToast("ล็อกอินไม่สำเร็จ กรุณาลองใหม่อีกครั้ง", duration: 1.3, position: .bottom)
                    })
                }
            }
        }
        task.resume()
        
    }
    
    // status = 11
    func submitConfirmDelete (){
        
        // data
        // nonokyo: set here
        let status  = "11"
        let data_bothdel = "\(status)+\(self.empID)"
        
        // url
        // nonokyo: set here
        let url = DataFactory.SUBMIT_API_ACCOUNT_DELETE_CONFIRM_URL
        
        let request:NSMutableURLRequest = NSMutableURLRequest(url: URL(string: url)!)
        request.httpMethod = "POST"
        request.addValue("multipart/form-data; boundary=X", forHTTPHeaderField: "Content-Type")
        
        // post data
        // nonokyo: set here
        let postData = DataFactory.generatePostData(
            [DataFactory.GeneratePostObject(name: "BothDel_data", value: EncryptDecryptAlgorithm.EncryptData(data_bothdel)),
             DataFactory.GeneratePostObject(name: "BothDelK", value: EncryptDecryptAlgorithm.getKeyEncrypt())]
        )
        // display sent data
        if DataFactory.DEBUG {
            print("url=\(request.url!)")
            print("data=\n\(postData)")
        }
        request.httpBody = postData.data(using: String.Encoding.utf8)
        
        DispatchQueue.main.async(execute: { () -> Void in
            self.loadingProgress.startAnimating()
        })
        
        let session = URLSession.shared
        let task = session.dataTask(with: request as URLRequest) { (data, response, error) -> Void in
            
            if error != nil {
                DispatchQueue.main.async(execute: { () -> Void in
                    self.loadingProgress.stopAnimating()
                    Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: DataString.error_connection_timeout)
                })
                return
            }
            
            var returnResult = 0
            var returnMessage = ""
            let result = NSString(data: data!, encoding: String.Encoding.utf8.rawValue)! as String
            
            if !result.isEmpty {
                // ========== RESPONSE UNKNOWN / DATA CORRUPTED ===========
                if result.hasPrefix("<!DOCTYPE") {
                    returnMessage = "Error: return data corrupted, please contact administrator."
                    returnResult = 0
                }
                // ========== RESPONSE ERROR =============
                else if result == "Login Fail.Try Again" {
                    returnMessage = "ล็อกอินไม่สำเร็จ กรุณาลองใหม่อีกครั้ง"
                    returnResult = 0
                }
                else if result == "PasswordBlank"{
                    returnMessage = "กรุณากรอกรหัสผ่าน"
                    returnResult = 0
                }
                // ========== RESPONSE SUCCESS ===========
                else if result == "Delete Person Complete" {
                    returnMessage = "ลบบัญชีสมบูรณ์"
                    returnResult = 1
                }
                    // ========== RESPONSE UNKNOWN / DATA CORRUPTED ===========
                else {
                    returnMessage = "Error: return data corrupted, please contact administrator. message: " + result
                    returnResult = 0
                }
            }
            else {
                returnMessage = "response == null"
                returnResult = 0
            }
            
            if returnResult == 1 {
                DispatchQueue.main.async(execute: { () -> Void in
                    self.loadingProgress.stopAnimating()
                    
                    // set segue to new page here
                    /*let alert = UIAlertController(title: "ลบบัญชีผู้ใช้งาน", message: "ลบบัญชีสมบูรณ์", preferredStyle: UIAlertController.Style.alert)
                    alert.addAction(UIAlertAction(title: DataString.button_ok, style: UIAlertAction.Style.default, handler:  { (alert: UIAlertAction!) in
                        self.performSegue(withIdentifier: "segue_remove_to_register1", sender: nil)
                    }))
                    self.present(alert, animated: true, completion: nil)*/
                    self.view.makeToast("ลบบัญชีสมบูรณ์", duration: 1.3, position: .center) { didTap in
                        if didTap {
                            //self.performSegue(withIdentifier: "segue_remove_to_register1", sender: nil)
                            self.performSegue(withIdentifier: "segue_remove_to_agreement", sender: nil)
                        } else {
                             //self.performSegue(withIdentifier: "segue_remove_to_register1", sender: nil)
                             self.performSegue(withIdentifier: "segue_remove_to_agreement", sender: nil)
                        }
                    }
                          
                })
            }
            else {
                DispatchQueue.main.async(execute: { () -> Void in
                    self.loadingProgress.stopAnimating()
                    Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: returnMessage)
                })
            }
        }
        task.resume()
        
    }
    
    /**
     part: etc
     */
    
    @IBAction func bRemoveClicked(_ sender: UIButton) {
        if stepRemove == 1 {
            self.view.endEditing(true)
            self.submitConfirmPassword()
        }
        else if stepRemove == 2 {

            self.view.endEditing(true)
            let alert = UIAlertController(title: "ลบบัญชีผู้ใช้งาน", message: "ลบบัญชีพนักงาน: \(self.empID) ออกจากระบบและอุปกรณ์นี้",preferredStyle: UIAlertController.Style.alert)
            
            alert.addAction(UIAlertAction(title: "ยกเลิก",
                                          style: UIAlertAction.Style.default,
                                          handler: {(_: UIAlertAction!) in
                                          self.performSegue(withIdentifier: "segue_exit_remove_to_checkin", sender: nil)
            }))
            //Cancel Action
            alert.addAction(UIAlertAction(title: "ตกลง",
                                          style: UIAlertAction.Style.default,
                                          handler: {(_: UIAlertAction!) in
                                            //Delete Account action
                                            self.submitConfirmDelete()
            }))
            self.present(alert, animated: true, completion: nil)
        }
    }
    
    @IBAction func bCloseClicked(_ sender: UIButton) {
        self.performSegue(withIdentifier: "segue_exit_remove_to_checkin", sender: nil)
    }
    
    override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?) {
        self.view.endEditing(true)
    }
    
    @objc func keyboardWillShow(notification: NSNotification) {
        if let keyboardSize = (notification.userInfo?[UIResponder.keyboardFrameEndUserInfoKey] as? NSValue)?.cgRectValue {
            if self.view.frame.origin.y == 0 {
                self.view.frame.origin.y -= keyboardSize.height
            }
        }
    }
    
    @objc func keyboardWillHide(notification: NSNotification) {
        if self.view.frame.origin.y != 0 {
            self.view.frame.origin.y = 0
        }
    }
}


