//
//  SigninController.swift
//  TIME
//
//  Created by Sruit Angkavanitsuk on 3/7/20.
//  Copyright © 2020 NSTDA. All rights reserved.
//

import UIKit

class SigninController: UIViewController {
    
    var isSignin = false
    var appDelegate:AppDelegate!
    
    @IBOutlet weak var ivLogo: UIImageView!
    @IBOutlet weak var vSignin: UIView!
    @IBOutlet weak var tfUsername: UITextField!
    @IBOutlet weak var tfPassword: UITextField!
    fileprivate var loadingProgress : UIActivityIndicatorView!
    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        appDelegate = UIApplication.shared.delegate as! AppDelegate
        if #available(iOS 13.0, *) {
            overrideUserInterfaceStyle = .light // force light mode
        }
        // Do any additional setup after loading the view.
        
        // loading view
        if DataFactory.BUSINESS_CODE == 1{
            ivLogo.image = UIImage(named: "i_nectec_main_logo2")
        }
        else if DataFactory.BUSINESS_CODE == 2 {
            ivLogo.image = UIImage(named: "i_nbtc_main_logo2")
        }
        
        if !isSignin {
            ivLogo.isHidden = false
            vSignin.isHidden = false
            
        }
        else {
            ivLogo.isHidden = true
            vSignin.isHidden = true

            DispatchQueue.main.async(execute: { () -> Void in
                self.performSegue(withIdentifier: "segue_signin_to_checkin", sender: nil)
            })
        }
        
        // loading animation
              loadingProgress = UIActivityIndicatorView() as UIActivityIndicatorView
              loadingProgress.frame = CGRect(x: 0, y: 0,
                  width: UIScreen.main.bounds.size.width,
                  height: UIScreen.main.bounds.size.height)
              loadingProgress.isOpaque = false
              loadingProgress.backgroundColor = UIColor( red: 0.95, green: 0.95, blue: 0.95, alpha: 0.5 )
              loadingProgress.hidesWhenStopped = true
              loadingProgress.style = UIActivityIndicatorView.Style.gray
              view.addSubview(loadingProgress)
    }
    
    override func viewWillAppear(_ animated: Bool) {
        NotificationCenter.default.addObserver(self, selector: #selector(keyboardWillShow), name: UIResponder.keyboardWillShowNotification, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(keyboardWillHide), name: UIResponder.keyboardWillHideNotification, object: nil)
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        NotificationCenter.default.removeObserver(self, name: UIResponder.keyboardWillShowNotification, object: self.view.window)
        NotificationCenter.default.removeObserver(self, name: UIResponder.keyboardWillHideNotification, object: self.view.window)
    }
    
    func submitSigninAuthentication (){
        
          // data
          // nonokyo: set here
          let macAdr = appDelegate.UUID
          var username = tfUsername.text!;
          var password = tfPassword.text!;
        
          if username.isEmpty {
            //Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: "กรุณากรอกชื่อบัญชีค่ะ")
            self.view.makeToast("ไม่สามารถล็อกอินได้ ลองใหม่อีกครั้ง", duration: 1.3, position: .bottom)
            return
         }
         else if password.isEmpty {
            //Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: "กรุณากรอกรหัสผ่านค่ะ")
            self.view.makeToast("ไม่สามารถล็อกอินได้ ลองใหม่อีกครั้ง", duration: 1.3, position: .bottom)
            return
         }
         else if password.length < 6 {
            //Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: "รหัสผ่านต้องมีความยาวอย่างน้อย 6 ตัวอักษรค่ะ")
            self.view.makeToast("ไม่สามารถล็อกอินได้ ลองใหม่อีกครั้ง", duration: 1.3, position: .bottom)
            return
         }
        
          var status = "6";
          var dataforth = status + "+" + username + "+" + password + "+" + macAdr;
          // url
          // nonokyo: set here
          let url = DataFactory.SUBMIT_API_SIGNIN_ACCOUNT_URL
          
          let request:NSMutableURLRequest = NSMutableURLRequest(url: URL(string: url)!)
          request.httpMethod = "POST"
          request.addValue("multipart/form-data; boundary=X", forHTTPHeaderField: "Content-Type")
          
          // post data
          // nonokyo: set here
          let postData = DataFactory.generatePostData(
              [DataFactory.GeneratePostObject(name: "data_forth", value: EncryptDecryptAlgorithm.EncryptData(dataforth)),
               DataFactory.GeneratePostObject(name: "data_forthK", value: EncryptDecryptAlgorithm.getKeyEncrypt()),
               DataFactory.GeneratePostObject(name: "data_password",value:EncryptDecryptAlgorithm.EncryptData(password)),
               DataFactory.GeneratePostObject(name: "data_passwordK", value: EncryptDecryptAlgorithm.getKeyEncrypt()),
               DataFactory.GeneratePostObject(name: "data_Mac",value:EncryptDecryptAlgorithm.EncryptData(macAdr)),
               DataFactory.GeneratePostObject(name: "data_MacK", value: EncryptDecryptAlgorithm.getKeyEncrypt())
            ]
          )
          // display sent data
          if DataFactory.DEBUG {
              print("url=\(request.url!)")
              print("data=\n\(postData)")
          }
          request.httpBody = postData.data(using: String.Encoding.utf8)
          
          DispatchQueue.main.async(execute: { () -> Void in
              self.loadingProgress.startAnimating()
          })
          
          let session = URLSession.shared
          let task = session.dataTask(with: request as URLRequest) { (data, response, error) -> Void in
              
              if error != nil {
                  DispatchQueue.main.async(execute: { () -> Void in
                      self.loadingProgress.stopAnimating()
                      Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: DataString.error_connection_timeout)
                  })
                  return
              }
              
              var returnResult = 0
              var returnMessage = ""
              let result = NSString(data: data!, encoding: String.Encoding.utf8.rawValue)! as String
              
              if !result.isEmpty {
                  // ========== RESPONSE UNKNOWN / DATA CORRUPTED ===========
                  if result.hasPrefix("<!DOCTYPE") {
                      returnMessage = "Error: return data corrupted, please contact administrator."
                      returnResult = 0
                  }
                  // ========== RESPONSE ERROR =============
                  else if result == "LoginFail" || result == "CannotLogin" {
                      //returnMessage = "ไม่สามารถเข้าใช้งานได้ ชื่อผู้ใช้หรือรหัสผ่านอาจผิดพลาด กรุณาลองอีกครั้ง หากไม่สำเร็จกรุณาแจ้งหน่วยงานของท่าน"
                     returnMessage = "ไม่สามารถล็อกอินได้ ลองใหม่อีกครั้ง"
                      returnResult = 0
                  }
                  else if result == "PasswordBlank"{
                      returnMessage = "กรุณากรอกรหัสผ่าน"
                      returnResult = 0
                  }
                  // ========== RESPONSE SUCCESS ===========
                  else if result == "LoginSuccessfully" {
                      returnResult = 1
                  }
                  // ========== RESPONSE UNKNOWN / DATA CORRUPTED ===========
                  else {
                      returnMessage = "Error: return data corrupted, please contact administrator. message: " + result
                      returnResult = 0
                  }
              }
              else {
                  returnMessage = "response == null"
                  returnResult = 0
              }
              
              if returnResult == 1 {
                  DispatchQueue.main.async(execute: { () -> Void in
                      self.loadingProgress.stopAnimating()
                      // set segue to new page here
                      // nonokyo: set here
                      self.performSegue(withIdentifier: "segue_signin_to_checkin", sender: nil)
                  })
              }
              else {
                  DispatchQueue.main.async(execute: { () -> Void in
                      self.loadingProgress.stopAnimating()
                      self.view.makeToast(returnMessage, duration: 3.0, position: .bottom)
                      //Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: returnMessage)
                  })
              }
          }
          task.resume()
    }
    
    @IBAction func bSigninClicked(_ sender: UIButton) {
        
        //start
        submitSigninAuthentication();
        
        /*DispatchQueue.main.async(execute: { () -> Void in
            self.performSegue(withIdentifier: "segue_signin_to_checkin", sender: nil)
        })*/
    }
    
    @IBAction func bCloseClicked(_ sender: UIButton) {
    }
    
    
    @IBAction func segue_exit_to_signin(_ segue: UIStoryboardSegue) {
        let segueName = segue.identifier
        if segueName == "segue_exit_captface_to_signin" {
            /*
            if let captFaceController = segue.source as? CaptureFaceController {
                //self.view.makeToast("ยืนยันตัวตนเรียบร้อย", duration: 3.0, position: .center)
            }
            */
        }
    }
    
    @IBAction func bFAQClicked(_ sender: UIButton) {
        /*var discTxt = "วัตถุประสงค์นี้ใช้เพื่อสำหรับการลงเวลาด้วยวิธีการตรวจสอบปัจจัยที่หลากหลายได้แก่ ใบหน้า รหัสผ่าน โทรศัพท์มือถือและสถานที่ ระบบนี้จะมีการบันทึกข้อมูลใบหน้าของผู้ใช้งาน และข้อมูลใบหน้าอาจถูกนำมาใช้เพื่อปรับปรุงระบบให้ดียิ่งขึ้นต่อไปในอนาคตและจะไม่มีการนำข้อมูลใบหน้าของผู้ใช้งานไปใช้ในวัตถุประสงค์อื่นนอกเหนือจากงานวิจัยภานใน ปัจจุบันระบบนี้ยังอยู่ระหว่างทดลองการใช้งาน"
        
        let alert = UIAlertController(title: "Disclamer", message: discTxt, preferredStyle: .alert)
        alert.addAction(UIAlertAction(title: "รับทราบ", style: .cancel, handler: nil))

        self.present(alert, animated: true)*/
        
        let urlComponents = URLComponents (string: "http://attime.secteam.in.th/#faq")!
        if #available(iOS 10.0, *) {
            UIApplication.shared.open(urlComponents.url!, options: [:], completionHandler: nil)
        } else {
            UIApplication.shared.openURL(urlComponents.url!)
        }
    }
    
    /**
    * Called when the user click on the view (outside the UITextField).
    */
    override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?) {
        self.view.endEditing(true)
    }

    @objc func keyboardWillShow(notification: NSNotification) {
        if let keyboardSize = (notification.userInfo?[UIResponder.keyboardFrameEndUserInfoKey] as? NSValue)?.cgRectValue {
            if self.view.frame.origin.y == 0 {
                self.view.frame.origin.y -= keyboardSize.height
            }
        }
    }

    @objc func keyboardWillHide(notification: NSNotification) {
        if self.view.frame.origin.y != 0 {
            self.view.frame.origin.y = 0
        }
    }
}

