//
//  UpdateController.swift
//  TIME
//
//  Created by Sruit Angkavanitsuk on 3/7/20.
//  Copyright © 2020 NSTDA. All rights reserved.
//

import UIKit

class UpdateController: UIViewController {
    
    @IBOutlet weak var ivLogo: UIImageView!
    @IBOutlet weak var lUsername: UILabel!
    @IBOutlet weak var tfPassword: UITextField!
    
    var appDelegate:AppDelegate!
    var idleCheck:DispatchWorkItem!
    
    fileprivate var loadingProgress : UIActivityIndicatorView!
    
    private var threeData = ""
    var empID = ""
    
    override func viewDidLoad() {
        super.viewDidLoad()
        appDelegate = UIApplication.shared.delegate as! AppDelegate
        if #available(iOS 13.0, *) {
            overrideUserInterfaceStyle = .light // force light mode
        }
        // Do any additional setup after loading the view.
        
        // loading view
        if DataFactory.BUSINESS_CODE == 1{
            ivLogo.image = UIImage(named: "i_nectec_update_logo")
        }
        else if DataFactory.BUSINESS_CODE == 2 {
            ivLogo.image = UIImage(named: "i_nbtc_update_logo")
        }
        
        lUsername.text = empID
        
        idleCheck = DispatchWorkItem(block: {
            // if stay in this page longer than 300 seconds
            // refresh in 300 sec
            DataFactory.IS_LOADING_FIRSTTIME = true
            self.performSegue(withIdentifier: "segue_exit_update_to_loading", sender: nil)
        })
        
        // loading animation
        loadingProgress = UIActivityIndicatorView() as UIActivityIndicatorView
        loadingProgress.frame = CGRect(x: 0, y: 0,
                                       width: UIScreen.main.bounds.size.width,
                                       height: UIScreen.main.bounds.size.height)
        loadingProgress.isOpaque = false
        loadingProgress.backgroundColor = UIColor( red: 0.95, green: 0.95, blue: 0.95, alpha: 0.5 )
        loadingProgress.hidesWhenStopped = true
        loadingProgress.style = UIActivityIndicatorView.Style.gray
        view.addSubview(loadingProgress)
    }
    
    override func viewWillAppear(_ animated: Bool) {
        NotificationCenter.default.addObserver(self, selector: #selector(keyboardWillShow), name: UIResponder.keyboardWillShowNotification, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(keyboardWillHide), name: UIResponder.keyboardWillHideNotification, object: nil)
        
        DispatchQueue.main.asyncAfter(deadline: .now() + 300, execute: idleCheck)
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        NotificationCenter.default.removeObserver(self, name: UIResponder.keyboardWillShowNotification, object: self.view.window)
        NotificationCenter.default.removeObserver(self, name: UIResponder.keyboardWillHideNotification, object: self.view.window)
        
        idleCheck.cancel()
    }
    
    /**
     part: connect API
     */
    
    // status = 3
    func submitConfirmPassword (){
        
        // data
        // nonokyo: set here
        let status = 3
        var password = tfPassword.text!;
        var business = ""
        
        if password.isEmpty {
            //Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: "กรุณากรอกรหัสผ่านค่ะ")
            self.view.makeToast("กรุณากรอกรหัสผ่าน", duration: 1.3, position: .bottom)
            return
        }
        else if password.length < 6 {
            //Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: "รหัสผ่านต้องมีความยาวอย่างน้อย 6 ตัวอักษรค่ะ")
            self.view.makeToast("ล็อกอินไม่สำเร็จ กรุณาลองใหม่อีกครั้ง", duration: 1.3, position: .bottom)
            return
        }
        
        if DataFactory.BUSINESS_CODE == 1 {
            business = "NECTEC"
        }
        else if DataFactory.BUSINESS_CODE == 2 {
            business = "NBTC"
        }
        threeData = "\(status)+\(empID)+\(password)+\(business)"
        // url
        // nonokyo: set here
        let url = DataFactory.SUBMIT_API_ACCOUNT_UPDATE_VERIFY_PASSWORD_URL
        
        let request:NSMutableURLRequest = NSMutableURLRequest(url: URL(string: url)!)
        request.httpMethod = "POST"
        request.addValue("multipart/form-data; boundary=X", forHTTPHeaderField: "Content-Type")
        
        // post data
        // nonokyo: set here
        let postData = DataFactory.generatePostData(
            [DataFactory.GeneratePostObject(name: "data_three", value: EncryptDecryptAlgorithm.EncryptData(threeData)),
             DataFactory.GeneratePostObject(name: "data_threeK", value: EncryptDecryptAlgorithm.getKeyEncrypt()),
             DataFactory.GeneratePostObject(name: "data_passwordUpdate",value:EncryptDecryptAlgorithm.EncryptData(password)),
             DataFactory.GeneratePostObject(name: "data_passwordUpdateK", value: EncryptDecryptAlgorithm.getKeyEncrypt())]
        )
        // display sent data
        if DataFactory.DEBUG {
            print("url=\(request.url!)")
            print("data=\n\(postData)")
        }
        request.httpBody = postData.data(using: String.Encoding.utf8)
        
        DispatchQueue.main.async(execute: { () -> Void in
            self.loadingProgress.startAnimating()
        })
        
        let session = URLSession.shared
        let task = session.dataTask(with: request as URLRequest) { (data, response, error) -> Void in
            
            if error != nil {
                DispatchQueue.main.async(execute: { () -> Void in
                    self.loadingProgress.stopAnimating()
                    Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: DataString.error_connection_timeout)
                })
                return
            }
            
            var returnResult = 0
            var returnMessage = ""
            let result = NSString(data: data!, encoding: String.Encoding.utf8.rawValue)! as String
            
            if !result.isEmpty {
                // ========== RESPONSE UNKNOWN / DATA CORRUPTED ===========
                if result.hasPrefix("<!DOCTYPE") {
                    returnMessage = "Error: return data corrupted, please contact administrator."
                    returnResult = 0
                }
                    // ========== RESPONSE ERROR =============
                else if result == "Login Fail.Try Again" {
                    returnMessage = "ล็อกอินไม่สำเร็จ กรุณาลองใหม่อีกครั้ง"
                    returnResult = 0
                }
                else if result == "PasswordBlank"{
                    returnMessage = "กรุณากรอกรหัสผ่าน"
                    returnResult = 0
                }
                    // ========== RESPONSE SUCCESS ===========
                else {
                    if self.empID == result {
                        returnResult = 1
                    }
                    else {
                        returnMessage = "Error: empID does not match with previous data."
                        returnResult = 0
                    }
                }
            }
            else {
                returnMessage = "response == null"
                returnResult = 0
            }
            
            if returnResult == 1 {
                DispatchQueue.main.async(execute: { () -> Void in
                    self.loadingProgress.stopAnimating()
                    
                    // set segue to new page here
                    /*let alert = UIAlertController(title: "อัพเดทบัญชีผู้ใช้งาน", message: "พนักงานบัญชีชื่อ: \(self.empID) ล็อกอินสำเร็จ", preferredStyle: UIAlertController.Style.alert)
                    alert.addAction(UIAlertAction(title: DataString.button_ok, style: UIAlertAction.Style.default, handler:  { (alert: UIAlertAction!) in
                        self.performSegue(withIdentifier: "segue_update_to_captprepare", sender: nil)
                    }))
                    self.present(alert, animated: true, completion: nil)*/
                     var message = "อัพเดทบัญชีผู้ใช้งาน พนักงานบัญชีชื่อ: \(self.empID) ล็อกอินสำเร็จ"
                    self.view.makeToast(message, duration: 2.0, position: .bottom) { didTap in
                       if didTap {
                           self.performSegue(withIdentifier: "segue_update_to_captprepare", sender: nil)
                       } else {
                            self.performSegue(withIdentifier: "segue_update_to_captprepare", sender: nil)
                       }
                   }
                })
            }
            else {
               if returnMessage.hasPrefix("Error:"){
                    DispatchQueue.main.async(execute: { () -> Void in
                        self.loadingProgress.stopAnimating()
                        Dialog.showError(self, title: DataString.TITLE_ERROR_ALERT, detail: returnMessage)
                    })
                }
                else {
                    DispatchQueue.main.async(execute: { () -> Void in
                        self.loadingProgress.stopAnimating()
                        self.view.makeToast("ล็อกอินไม่สำเร็จ กรุณาลองใหม่อีกครั้ง", duration: 1.3, position: .bottom)
                    })
                }
            }
        }
        task.resume()
    }
    
    /**
     part: etc
     */
    
    @IBAction func bUpdateClicked(_ sender: UIButton) {
        //start
        submitConfirmPassword()
        /*DispatchQueue.main.async(execute: { () -> Void in
            self.performSegue(withIdentifier: "segue_update_to_captprepare", sender: nil)
        })*/
    }
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "segue_update_to_captprepare" {
            let capturePrepareController = segue.destination as! CapturePrepareController
            capturePrepareController.empID = empID;
            capturePrepareController.threeData = threeData
            capturePrepareController.from_code = "UPDATE"
        }
    }
    
    @IBAction func bCloseClicked(_ sender: UIButton) {
        self.performSegue(withIdentifier: "segue_exit_update_to_checkin", sender: nil)
    }
    
    @IBAction func segue_exit_to_update(_ segue: UIStoryboardSegue) {
    }
    
    /**
     * Called when the user click on the view (outside the UITextField).
     */
    override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?) {
        self.view.endEditing(true)
    }
    
    @objc func keyboardWillShow(notification: NSNotification) {
        if let keyboardSize = (notification.userInfo?[UIResponder.keyboardFrameEndUserInfoKey] as? NSValue)?.cgRectValue {
            if self.view.frame.origin.y == 0 {
                self.view.frame.origin.y -= keyboardSize.height
            }
        }
    }
    
    @objc func keyboardWillHide(notification: NSNotification) {
        if self.view.frame.origin.y != 0 {
            self.view.frame.origin.y = 0
        }
    }
}


