//
//  EncryptData.swift
//  TIME
//
//  Created by Sruit Angkavanitsuk on 3/7/20.
//  Copyright © 2020 NSTDA. All rights reserved.
//

import Foundation
import CryptoSwift
import SwiftyRSA

struct EncryptDecryptAlgorithm {
    static var _CHAR = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    static var privateKeyRaw = ""
    static var privateKeyEncrypt = ""
    
    static var RSA_public_key = "MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQCoNwD7J0HS94YcPlRon1oZHLpmEhkOJZikbhyk0XHJ1+kjHxaM+N10nPG2FWNafvW/Jr/7DYqwaF7Ub/GLevWuSZ/6iL8dZicEv+4dlxzDJ0+lkOSgEl1K0O4K5bAQ3Z/g/ZP53NQKhlpgdofWPDNhCSSSa/HrOrt4OAOYZGp8iwIDAQAB"
    
    static func EncryptData( _ sdata: String) -> String {
        // generate Key
        if privateKeyRaw.isEmpty {
            generateKey()
        }
        
        // aes cipher
        do {
            // CryptoSwift
            // Notice regarding padding: Manual padding of data is optional, and CryptoSwift is using PKCS7 padding by default. If you need to manually disable/enable padding, you can do this by setting parameter for AES class
            // convert [UInt8] to [Int8]
            let keyByteUnsigned = privateKeyRaw.bytes.sha256() // iOS must use this value
            //let keyByteSigned = keyByteUnsigned.map { Int8(bitPattern: $0) } // Android will show/display this value
            //print("keyByteUnsigned=\(keyByteUnsigned)") // iOS
            //print("keyByteSigned=\(keyByteSigned)") // Android
            
            let ivByte:[UInt8] = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16] // no used
            let aes = try AES(key: privateKeyRaw.bytes.sha256(),
                              blockMode: CBC(iv: ivByte),
                              padding: .pkcs5)
            
            let cipherByte = try aes.encrypt(ivByte + Array(sdata.utf8))
            //print("cipherByte=\(cipherByte)")
            let cipherText = cipherByte.toBase64()!
            //print("cipherText=\(cipherText)")
            
            // reverse
            let cipherTextReverse = String(cipherText.reversed())
            //print("cipherTextReverse=\(cipherTextReverse)")
            
            //convert to base64 round 1
            let _cipherTextReverseLineFeed = [10] + Array(cipherTextReverse.utf8) // [10] = line feed, iOS did not print
            //let x1Signed = x1Unsigned.map { Int8(bitPattern: $0) } // Android will show/display this value
            //print("_cipherTextReverseLineFeed=\(_cipherTextReverseLineFeed)") // iOS
            //print("x1S=\(x1Signed)") // Android
            
            let cipherTextReverseBase64 = _cipherTextReverseLineFeed.toBase64()
            
            //Upper to Lower & Lower to Upper
            let cipherTextReverseBase64FlipCase = swapCase(cipherTextReverseBase64!)
            //print("cipherTextReverseBase64FlipCase=\(cipherTextReverseBase64FlipCase)")
            
            return cipherTextReverseBase64FlipCase
        }
        catch {
            return "ERROR: ENCRYPT DATA FAILED, MESSAGE: \(error)"
        }
    }
    
    static func getKeyRaw() -> String {
        if privateKeyRaw.isEmpty {
            generateKey()
        }
        return privateKeyRaw
    }
    
    static func getKeyEncrypt() -> String {
        if privateKeyRaw.isEmpty {
            generateKey()
        }
        return privateKeyEncrypt
    }
    
    private static func generateKey() -> String {
        let dateFormat = DateFormatter()
        dateFormat.calendar = Calendar(identifier: Calendar.Identifier.gregorian)
        dateFormat.dateFormat = "yyyy-M-d"
        
        //privateKeyRaw = "MMWSGKYIKF+2020-7-13" // sample
        privateKeyRaw = "\(getRandomString(10))+\(dateFormat.string(from: Date()))"
        print("privateKeyRaw=\(privateKeyRaw)")
        
        privateKeyEncrypt = EncryptKey()
        
        return privateKeyRaw
    }
    
    private static func EncryptKey() -> String {
        do {
            let RSAbase64DataUnsigned = Data(base64Encoded: RSA_public_key)
            //let RSAbase64DataSigned = [UInt8](RSAbase64DataUnsigned!).map { Int8(bitPattern: $0) } // Android will
            //print("RSAbase64DataUnsigned!.bytes=\(RSAbase64DataUnsigned!.bytes)") // iOS
            //print("RSAbase64DataSigned!.bytes=\(RSAbase64DataSigned)") // Android
            
            let publicKey = try PublicKey(data: RSAbase64DataUnsigned!)
            let clear = try ClearMessage(string: privateKeyRaw, using: .utf8)
            let encrypted = try clear.encrypted(with: publicKey, padding: .PKCS1)
            
            //print("encrypted=\(encrypted.data)")
            //print("encrypted=\(encrypted.base64String)")
            
            let cipherKey = encrypted.base64String
            
            // reverse
            let cipherKeyReverse = String(cipherKey.reversed())
            //print("cipherKeyReverse=\(cipherKeyReverse)")
            
            //convert to base64 round 1
            let _cipherKeyReverseLineFeed = [10] + Array(cipherKeyReverse.utf8) // [10] = line feed, iOS did not print
            //let x1Signed = x1Unsigned.map { Int8(bitPattern: $0) } // Android will show/display this value
            //print("_cipherKeyReverseLineFeed=\(_cipherKeyReverseLineFeed)") // iOS
            //print("x1S=\(x1Signed)") // Android
            
            //let x2 = Data(cipherTextReverse.utf8).base64EncodedString()
            let cipherKeyReverseBase64 = _cipherKeyReverseLineFeed.toBase64()
            
            //Upper to Lower & Lower to Upper
            let cipherKeyReverseBase64FlipCase = swapCase(cipherKeyReverseBase64!)
            //print("cipherKeyReverseBase64FlipCase=\(cipherKeyReverseBase64FlipCase)")
            
            return cipherKeyReverseBase64FlipCase
        }
        catch {
            return "ERROR: ENCRYPT KEY FAILED, MESSAGE: \(error)"
        }
    }
    
    // reverse byte array
    private static func swapUInt8Data(data : Data) -> Data {
        var mdata = data // make a mutable copy
        let count = data.count / MemoryLayout<UInt8>.size
        mdata.withUnsafeMutableBytes { (i16ptr: UnsafeMutablePointer<UInt8>) -> Void in
            for i in 0..<count {
                i16ptr[i] =  i16ptr[i].byteSwapped
            }
        }
        return mdata
    }
    
    private static func swapCase(_ str : String) -> String {
        var result = ""
        for c in str {
            let s = String(c)
            let lo = s.lowercased()
            let up = s.uppercased()
            result += (s == lo) ? up : lo
        }
        return result
    }
    
    private static func getRandomString(_ length: Int) -> String {
        return String((0..<length).map{ _ in _CHAR.randomElement()! })
    }
    
}
